# Copyright (c) 2018 exstnce
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# File: core.rb
# Project: dotfiles-cli
# Author: exstnce (exstnce@protonmail.ch)
# Created on Sunday, 8th July 2018 5:15:53 pm

require 'git'
require 'colorize'

require_relative 'utils/fs'
require_relative 'utils/get'
require_relative 'utils/permissive'
require_relative 'core/install'
require_relative 'core/response'

module Core
  include Get
  include Sudo
  include FileSystem
  include Install
  include Response

  @config_is_empty = Dir.entries(@@config).size <= 2

  def self.help(command=nil)
    if command == nil
      Response.help
      exit
    end

    case command.downcase
      when 'install'
        Response.install
      when 'use'
        Response.use
      when 'save'
        Response.save
      when 'delete'
        Response.delete
      else
        Response.help
    end
    exit
  end
  
  def self.install(link)
    name = Get.name
    path = Get.path(name)

    begin
      Git.clone(link, path)
    rescue Exception => e
      puts "#{"Error".red}: Could not clone the repository"
      puts
      puts e.message
      exit
    end

    puts "#{name.cyan} #{"successfully installed".green}"
    puts "Path: #{path}"

    print 'Do you want to use the installed dotfiles? (Y/n) '
    use = Get.answer

    if use
      use(name)
    end
  end

  def self.use(name)
    path = Get.exists(name)

    print 'Backup current dotfiles? (Y/n) '
    save = Get.answer

    if save
      save()
    end

    scan(name, path)
  end

  def self.save
    name = Get.name
    path = Get.path(name)

    init(path)

    config(path)
    tools(path)

    print 'Do you also want to backup your /etc directory? (Y/n) '
    backup = Get.answer
    
    if backup
      etc(path)
    end
  end

  def self.delete(name)
    path = Get.exists(name)

    FileUtils.rm_rf(path)
  end

  private

  def self.init(path)
    Dir.mkdir(path)
    Dir.mkdir(path+'config/')
    Dir.mkdir(path+'tools/')
  end

  def self.config(path)
    contents = FileSystem.contents(@@config)
    path+='config/'

    Sudo.copy(contents, path)
  end

  def self.tools(path)
    # Returns an array of ~/.* files
    contents = FileSystem.rglob(@@root)
    path+='tools/'

    contents.each do |file|
      Sudo.copy(file, path)
    end
  end

  def self.etc(path)
    contents = FileSystem.contents(@@etc)
    path+='etc/'

    Dir.mkdir(path)
    Sudo.copy(contents, path)
  end

  def self.scan(name, path)
    # ~/.dotfiles/name/etc
    etc = path+'etc/'
    if File.exists?(etc)
      Install.config(etc)
    end

    # ~/.dotfiles/name/config
    config = path+'config/'
    if File.exists?(config)
      Install.config(config)
    end

    # ~/.dotfiles/name/tools
    tools = path+'tools/'
    if File.exists?(tools)
      Install.tools(tools)
    end

    # ~/.dotfiles/name/scripts
    scripts = path+'scripts/'
    if File.exists?(scripts)
      Install.scripts(name, scripts)
    end

    # ~/.dotfiles/name/wallpapers
    wallpapers = path+'wallpapers/'
    if File.exists?(wallpapers)
      Install.wallpapers(name, wallpapers)
    end
  end
end